﻿/*
    Little Registry Optimizer
    Copyright (C) 2008-2009 Little Apps (http://www.littleapps.co.cc/)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

using System;
using System.Collections.Generic;
using System.Threading;
using System.Windows.Forms;
using System.IO;

namespace Little_Registry_Optimizer
{
    static class Program
    {
        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main()
        {
            bool bNewMutex = false;

            using (Mutex mutex = new Mutex(true, "Little Registry Optimizer", out bNewMutex))
            {
                if (!bNewMutex)
                {
                    MessageBox.Show("Another program is already running...", Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }

                string strCommonProgFiles = Environment.GetFolderPath(Environment.SpecialFolder.CommonProgramFiles);

                // Check if backup directory exists
                if (string.IsNullOrEmpty(Properties.Settings.Default.strBackupFolder))
                {
                    Properties.Settings.Default.strBackupFolder = string.Format("{0}\\{1}\\Backup", strCommonProgFiles, Application.ProductName);

                    if (!Directory.Exists(Properties.Settings.Default.strBackupFolder))
                        Directory.CreateDirectory(Properties.Settings.Default.strBackupFolder);
                }

                if (string.IsNullOrEmpty(Properties.Settings.Default.strErrorFolder))
                {
                    Properties.Settings.Default.strErrorFolder = string.Format("{0}\\{1}\\Errors", strCommonProgFiles, Application.ProductName);

                    if (!Directory.Exists(Properties.Settings.Default.strErrorFolder))
                        Directory.CreateDirectory(Properties.Settings.Default.strErrorFolder);
                }

                // Add event handler for thread exceptions
//#if (!DEBUG)
                AppDomain.CurrentDomain.UnhandledException += new UnhandledExceptionEventHandler(CurrentDomain_UnhandledException);
                Application.ThreadException += new ThreadExceptionEventHandler(Application_ThreadException);
//#endif

                Permissions.SetPrivileges(true);

                Application.EnableVisualStyles();
                Application.SetCompatibleTextRenderingDefault(false);
                Application.Run(new Main());

                Permissions.SetPrivileges(false);

                Properties.Settings.Default.Save();
            }
        }

        static void CurrentDomain_UnhandledException(object sender, UnhandledExceptionEventArgs e)
        {
            CrashReporter ErrorDlg = new CrashReporter((Exception)e.ExceptionObject);
            ErrorDlg.ShowDialog();
        }

        static void Application_ThreadException(object sender, ThreadExceptionEventArgs e)
        {
            CrashReporter ErrorDlg = new CrashReporter(e.Exception);
            ErrorDlg.ShowDialog();
        }
    }
}
