﻿/*
    Little Registry Optimizer
    Copyright (C) 2008-2009 Little Apps (http://www.littleapps.co.cc/)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Threading;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Microsoft.Win32;

namespace Little_Registry_Optimizer
{
    public partial class SplashScreen : Form
    {
        public SplashScreen()
        {
            InitializeComponent();
        }

        public static void ShowSplashScreen()
        {
            SplashScreen frmSplashScreen = new SplashScreen();
            frmSplashScreen.ShowDialog();
        }

        private void SplashScreen_Shown(object sender, EventArgs e)
        {
            // Call Refresh so dialog is displayed
            this.Refresh();

            // Create restore point if program is started for 1st time
#if (!DEBUG)
            if (Main.LastStart == DateTime.MinValue)
            {
                long lRestoreSeqNum = 0;

                if (SysRestore.StartRestore("Before Little Registry Cleaner First Run", SysRestore.RestoreType.FirstRun, out lRestoreSeqNum) == 0) 
                    SysRestore.EndRestore(lRestoreSeqNum);
            }
#endif

            // Enumerate through hivelist
            RegistryKey rkHives = null;
            using (rkHives = Registry.LocalMachine.OpenSubKey(@"SYSTEM\CurrentControlSet\Control\hivelist"))
            {
                if (rkHives == null)
                    throw new Exception("Unable to open hivelist... this can be a problem!");

                foreach (string strValueName in rkHives.GetValueNames())
                {
                    string strHivePath = rkHives.GetValue(strValueName) as string;
                    if (!string.IsNullOrEmpty(strValueName) && !string.IsNullOrEmpty(strHivePath))
                        Main.arrHives.Add(new Hive(strValueName, strHivePath));
                }
            }

            // Check for update if it has been over 2 weeks
            if (DateTime.Now.Subtract(Main.LastUpdateCheck).TotalDays >= 14)
            {
                Main.LastUpdateCheck = DateTime.Now.Date;

                string strVersion = "", strChangeLogURL = "", strDownloadURL = "", strReleaseDate = "";
                if (Utils.FindUpdate(ref strVersion, ref strReleaseDate, ref strChangeLogURL, ref strDownloadURL))
                    if (MessageBox.Show(this, "A newer version is available. Would you like to download it?", Application.ProductName, MessageBoxButtons.YesNo, MessageBoxIcon.Information) == DialogResult.Yes)
                        Utils.LaunchURI(strDownloadURL);
            }

            // Close dialog once this is finished
            this.DialogResult = DialogResult.OK;

            Thread.Sleep(1000);
        }

        private void SplashScreen_DoubleClick(object sender, EventArgs e)
        {
            if (this.DialogResult == DialogResult.OK)
                this.Close();
        }
    }
}
